/*
        FILE :                  Main.cpp

        DESCRIPTION :           Main file of Moteur3D

        AUTHOR :                Trax / Aspirine (c), 1999

        VERSION :               0.1.0b
*/

//// INCLUDES /////////////////////////////////////////////////////////////////

#include <Application.h>
#include <WindowScreen.h>
#include <Screen.h>
#include <Errors.h>
#include <Entry.h>
#include <Path.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

// 3D engine includes
#include "3dstruct.h" // Defini les differentes structures
#include "loadasc.h"  // 3d studio (.asc) file reader
#include "vector.h"   // vector routines
#include "matrix.h"   // matrix routines
#include "object.h"  // object routines
#include "polygon.h"  // polygon routines
#include "pcx.h"      // pcx routines
#include "light.h"    // light routines
#include "scene.h"    // scenes routines
#include "camera.h"   // camera routines
#include "texture.h"  // texture routines

//// DEFINES /////////////////////////////////////////////////////////////////

#define XRES		640
#define YRES		480
#define VIDEOMODE 	B_32_BIT_640x480

//// GLOBALS /////////////////////////////////////////////////////////////////

unsigned int 	*buffer;	// these vars are used by polygon.cpp
int 			xres;		// X resolution
BPath 			the_path;	// path to the application

//// CLASSES /////////////////////////////////////////////////////////////////

class Moteur3D : public BApplication
{
public:
 	Moteur3D(void);
	virtual void ReadyToRun(void);
};

class DisplayWin : public BWindowScreen
{
	private:
		uint32* frame_buffer;	// pointer to screen buffer
		uint32* tmp_screen;
	public:
		DisplayWin(status_t*);
	private:
		DisplayWin(const DisplayWin& copy);
		DisplayWin& operator = (const DisplayWin& assign);
		void MakePath(char *filename, char *name);
		void ScreenConnected(bool);
};		

//// METHODS ////////////////////////////////////////////////////////////////

Moteur3D::Moteur3D() : BApplication("application/x-vnd.Moteur3D")
{
	status_t error=B_ERROR;
	
	DisplayWin *wnd = new DisplayWin(&error);

	if((error < B_OK) || (wnd == NULL))
		PostMessage(B_QUIT_REQUESTED);
}

void Moteur3D::ReadyToRun(void)
{
	// get the application directory
	app_info the_info;
	GetAppInfo(&the_info);
	BEntry the_file(&the_info.ref);
	BEntry the_dir;
	the_file.GetParent(&the_dir);
	the_dir.GetPath(&the_path);
}


DisplayWin::DisplayWin(status_t* error) : BWindowScreen("TestWindow", VIDEOMODE, error, false)
{
	Show();
}

void DisplayWin::MakePath(char *filename, char *name)
{
	strcpy(filename, the_path.Path());
	strcat(filename, "/");
	strcat(filename, name);
}

void DisplayWin::ScreenConnected(bool connected)
{
	int iters;
	double mypi  = M_PI;
	char filename[256];
		
	// 3d engine vars
    float krx, kry, krz, right = 0, down = 0, forward = 0;
    SCENE scene;
    CAMERA camera;
    int twotorus, textlogo, sphere, bigR, tinytorus1, tinytorus2; 
		
	if(connected)
	{
		// get screen buffer
		frame_buffer=(uint32*)(CardInfo()->frame_buffer);
		// clear screen buffer
		memset(frame_buffer, 0, XRES*YRES*4 );
		// allocate one back buffer 
		tmp_screen = new uint32 [YRES*XRES];
		buffer = (unsigned int *)tmp_screen;
		xres = XRES;

		// use normat texture rendering
		scene.perspective = 0; 

        // Load textures
        MakePath(filename, "metal.pcx");
        load_textmap(&scene, 0, filename);
        MakePath(filename, "s_ashsen.pcx");
        load_textmap(&scene, 1, filename);
        MakePath(filename, "altaqua.pcx");
        load_textmap(&scene, 2, filename);

        // set max objects
        obj_loader_init(&scene, 8);

		// Load 3d objects
        MakePath(filename, "3torus.asc");
        twotorus = new_obj(&scene, filename, 5.0, 0.0, 0.0, 0.0,
                                 p_texture + p_hfre, 2, 0, 0, 0);

        MakePath(filename, "arcee.asc");
        textlogo = new_obj(&scene, filename, 0.4, 250.0, 10.0, 500.0,
                                      p_flat + p_hfre, 0, 0, 0, 255);

        MakePath(filename, "pallero.asc");
        sphere = new_obj(&scene, filename, 0.4, 250.0, 0.1, 0.1,
                            p_texturegouraud + p_hfre, 1, 0, 0, 0);

        MakePath(filename, "ar2.asc");
        bigR = new_obj(&scene, filename, 0.4, -120.0 , 0.1, 0.1,
                     p_envmap + p_hfre, 0, 0, 0, 0);

        MakePath(filename, "toorus.asc");
        tinytorus1 =  new_obj(&scene, filename, 0.4, 0.0 , 0.0, 0.0,
                          p_phong + p_hfre, 0, 255, 15, 15);

        tinytorus2 = new_obj(&scene, filename, 0.8, 0.0 , 100.0, -100.0,
                          p_gouraud + p_hfre, 0, 15, 255, 15);

        all_objects_loaded(&scene);

        // Set light position
        light_set(&scene.light, 0, 0, -1);

        // Init camera
        init_camera(&camera, 256.0, 159.0, 99.0, 
             (float)XRES / 320.0f,  // x_suhde
             (float)YRES / 200.0f, // y_suhde
             0, XRES,  
             0, YRES);

		right = down = forward = krx = kry = krz = 0;

		for(iters = 0; iters < 200; iters++)
		{
                // update camera
                camera_update(&camera, krx, kry, krz, right, down, forward);

                // prepare frame
                prepare_frame(&scene);

                // animate objects
                object_update(&scene, twotorus,  -2 * mypi / 100, -2 * mypi / 100, -2 * mypi / 100, 0, 0, 0);
                object_update(&scene, sphere,  2 * mypi / 100, 2 * mypi / 100, 2 * mypi / 100, 0, 0, 0);
                object_update(&scene, textlogo, 2 * mypi / 150, 0, 0, 0, 0, 0);
                object_update(&scene, bigR, 0, 2 * mypi / 130, 2 * mypi / 150, 0, 0, 0);
                object_update(&scene, tinytorus1, 2 * mypi / 200, 2 * mypi / 200, 2 * mypi / 200, 0, 0, 0);
                object_update(&scene, tinytorus2, 2 * mypi / 200, 2 * mypi / 200, 2 * mypi / 200, 0, 0, 0);

                // rotate objects
                putobject(&scene, twotorus, &camera);
                putobject(&scene, textlogo, &camera);
                putobject(&scene, sphere, &camera);
                putobject(&scene, bigR, &camera);
                putobject(&scene, tinytorus1, &camera);
                putobject(&scene, tinytorus2, &camera);

                // render view
                draw(&scene, &camera);

				// flip buffer to screen
				BScreen(this).WaitForRetrace();
				memcpy(frame_buffer, tmp_screen, XRES*YRES*4);

				// clear buffer
				memset(tmp_screen, 0, XRES*YRES*4);

				// move backward
				if (iters > 100)
					forward --;
/*
				How to do something like ? With GetKeyInfo ???
				case 'z':down += 14; break;
				case 'a':down -= 14; break;
				case 'c':kry -= 2 * mypi / 150; break;
				case 'x':kry +=	 2 * mypi / 150; break;
				case 'f':krx += 2 * mypi / 150; break;
				case 'r':krx -= 2 * mypi / 150; break;
				case 't':krz += 2 * mypi / 150; break;
				case 'y':krz -= 2 * mypi / 150; break;
				case 'q':right -= 10; break;
				case 'w':right += 10; break;
				case 'e':forward += 20; break;
				case 'd':forward -= 20; break;
				case '+':scene.perspective = 1; break;
				case '-':scene.perspective = 0; break;
*/

		}

        // free textures
        free_textmap(&scene, 0);
        free_textmap(&scene, 1);
        free_textmap(&scene, 2);

        // uninit objects
        dispose_object(&scene, bigR);
        dispose_object(&scene, tinytorus1);
        dispose_object(&scene, tinytorus2);
        dispose_object(&scene, sphere);
        dispose_object(&scene, textlogo);
        dispose_object(&scene, twotorus);

        // uninit scene
        free_object_structs(&scene);

		// exit app
		be_app->PostMessage(B_QUIT_REQUESTED);
	}
}


int main(void)
{
	Moteur3D my_app;
	
	my_app.Run();
	return B_OK;
}
